
   Modification History:
   [1.0]    29.06.2009    Tobias Pluess
   - first release

   [1.1]    09.09.2009    Tobias Pluess
   - added a general interrupt dispatcher
   - added a handler for software interrupts

   [1.2]    28.10.2009    Tobias Pluess
   - added a general trap procedure: the user can call a specific c-style
     function with a parameter, and a swi with that parameter is generated.
     this is the only way to change the mode from user to supervisor mode.
 *****************************************************************************/

/******************************************************************************
 * INCLUDE FILES
 *****************************************************************************/

/******************************************************************************
 * PRIVATE FUNCTION PROTOTYPES (STATIC)
 *****************************************************************************/

/******************************************************************************
 * PRIVATE VARIABLES
 *****************************************************************************/

/******************************************************************************
 * OTHER DEFINITIONS
 *****************************************************************************/

        module ?cstartup

        section IRQ_STACK:DATA:NOROOT(3)
        section FIQ_STACK:DATA:NOROOT(3)
        section SVC_STACK:DATA:NOROOT(3)
        section ABT_STACK:DATA:NOROOT(3)
        section UND_STACK:DATA:NOROOT(3)
        section CSTACK:DATA:NOROOT(3)
        section TEMPSTACK:DATA:NOROOT(3)

MODE_USR define 0x10 /* user mode */
MODE_FIQ define 0x11 /* fiq mode */
MODE_IRQ define 0x12 /* interrupt mode */
MODE_SVC define 0x13 /* supervisor mode */
MODE_ABT define 0x17 /* abort mode */
MODE_UND define 0x1B /* undefinied instruction */
MODE_SYS define 0x1F /* system mode */

I_BIT define (1 << 7) /* interrupt enable */
F_BIT define (1 << 6) /* fiq enable */
T_BIT define (1 << 5) /* thumb mode */

VICINTENCLR define 0xFFFFF014 /* vic interrupt clear register */
VICSOFTINTCLR define 0xFFFFF01C /* vic software interrupt clear register */
VICADDRESS define 0xFFFFFF00 /* vic vector register */

FIO0DIR define 0x3FFFC000
FIO0CLR define 0x3FFFC01C

        section .intvec:CODE:NOROOT(2)
        public __vector
        arm

/******************************************************************************
 * MODULE FUNCTIONS (PUBLIC)
 *****************************************************************************/

__vector:
        ldr pc, =reset_vector /* branch to reset handler */
        ldr pc, =undef_vector /* undefinied instruction */
        ldr pc, =swi_vector /* software interrupt */
        ldr pc, =pabort_vector /* prefetch abort */
        ldr pc, =dabort_vector /* data abort */
        nop /* reserved vector; downloader places vector checksum here */
        ldr pc, =irq_vector /* interrupt */
        ldr pc, =fiq_vector /* fast interrupt */

        section .text:CODE:NOROOT(2)

        require __vector
        public __iar_program_start
        public fDisableIRQ
        public fEnableIRQ
        public fDisableFIQ
        public fEnableFIQ
        public fTrap
        public htons
        public ntohs
        public htonl
        public ntohl
        extern fLowLevelInit
        extern fDispatchSWI
        extern __iar_data_init2
        extern main

        arm /* always in arm mode after reset */

/* first, loop for a while so that the debugger can halt the cpu befor any
   other instructions are executed. */

reset_vector:
__iar_program_start:
?cstartup:
        ldr r1, =FIO0CLR
        mov r2, #(1 << 25)
        str r2, [r1] /* clear p0.25 */
        ldr r1, =FIO0DIR
        mov r2, #(1 << 25) /* p0.25 is an output (piezo) */
        str r2, [r1]

        mov r0, #(1 << 18) /* waste a few cycles */
delay:
        sub r0, r0, #1
        cmp r0, #0
        bne delay

        /* create a temporary stack in the internal ram for fLowLeevelInit */
        ldr sp, =sfe(TEMPSTACK)
        mov r0, #1
        ldr r12, =fLowLevelInit
        mov lr, pc
        bx r12

        /* switch to UND mode and initialize its stack pointer */
        msr cpsr_c, #MODE_UND | I_BIT | F_BIT
        ldr sp, =sfe(UND_STACK)

        /* switch to ABT mode and initialize its stack pointer */
        msr cpsr_c, #MODE_ABT | I_BIT | F_BIT
        ldr sp, =sfe(ABT_STACK)

        /* switch to FIQ mode and initialize its stack pointer */
        msr cpsr_c, #MODE_FIQ | I_BIT | F_BIT
        ldr sp, =sfe(FIQ_STACK)

        /* switch to IRQ mode and initialize its stack ponter */
        msr cpsr_c, #MODE_IRQ | I_BIT | F_BIT
        ldr sp, =sfe(IRQ_STACK)

        /* switch to SVC mode and initialize its stack pointer */
        msr cpsr_c, #MODE_SVC | I_BIT | F_BIT
        ldr sp, =sfe(SVC_STACK)

        /* clear all pending interrupts */
        ldr r1, =VICINTENCLR
        mov r2, #0xFFFFFFFF
        str r2, [r1]

        /* clear all pending software interrupts */
        ldr r1, =VICSOFTINTCLR
        mov r2, #0xFFFFFFFF
        str r2, [r1]

        /* switch to USR mode and initialize its stack pointer */
        msr cpsr_c, #MODE_USR | I_BIT | F_BIT /* start in user mode */
        ldr sp, =sfe(CSTACK)

        /* initialization routine, which loads variable initial values and
        overwrites static variables with zero. */
        ldr r12, =__iar_data_init2
        mov lr, pc
        bx r12

        /* when initialization is done, jump to the main program after all
        registers have been cleared. */
        eor r0, r0, r0
        eor r1, r1, r1
        eor r2, r2, r2
        eor r3, r3, r3
        eor r4, r4, r4
        eor r5, r5, r5
        eor r6, r6, r6
        eor r7, r7, r7
        eor r8, r8, r8
        eor r9, r9, r9
        eor r10, r10, r10
        eor r11, r11, r11
        ldr r12, =main
        mov lr, pc
        bx r12

exit:
        b exit

undef_vector:
        b undef_vector

swi_vector:
        stmfd sp!, {r4, r5, lr} /* save registers on the stack */
        mrs r5, spsr
        ldr r4, [lr, #-4] /* get the swi instruction code */
        bic r4, r4, #0xff000000 /* mask out the opcode and get the parameter */
        cmp r4, #((switable_end - switable_start) / 4) - 1
        ldrls pc, [pc, r4, lsl #2] /* branch to one of the 4 handlers */
        b swi_user /* or go to the user swi handler */

switable_start:
        dcd disable_irq
        dcd enable_irq
        dcd disable_fiq
        dcd enable_fiq
switable_end:

/* swi #0 -> disable irq */
disable_irq:
        mov r0, r5
        orr r5, r5, #I_BIT /* set the i bit */
        b swi_exit

/* swi #1 -> enable irq */
enable_irq:
        mov r0, r5
        bic r5, r5, #I_BIT /* clear the i bit */
        msr spsr_c, r4
        b swi_exit

/* swi #2 -> disable fiq */
disable_fiq:
        mov r0, r5
        orr r5, r5, #F_BIT /* set the f bit */
        b swi_exit

/* swi #3 -> enable fiq */
enable_fiq:
        mov r0, r5
        bic r5, r5, #F_BIT /* clear the f bit */
        b swi_exit

/* swi handler for user swi functions */
swi_user:
        stmfd sp!, {r1-r12} /* save the registers on the stack */
        ldr r12, =fDispatchSWI
        mov r0, r4 /* place the swi parameter in r0 */
        mov lr, pc
        bx r12 /* branch to the swi dispatcher */
        ldmfd sp!, {r1-r12} /* return */

swi_exit:
        msr spsr_c, r5
        ldmfd sp!, {r4, r5, pc}^

pabort_vector:
        b pabort_vector

dabort_vector:
        b dabort_vector

irq_vector:
        sub lr, lr, #4 /* calculate the address of the next instruction */
        stmfd sp!, {lr} /* save it on the stack */
        mrs lr, spsr /* load the spsr in the lr */
        stmfd sp!, {lr} /* save the spsr on the stack */
        msr cpsr_c, #MODE_SVC | I_BIT /* switch to supervisor mode */
        stmfd sp!, {r0-r3, r12, lr} /* save registers on the stack */
        mov r12, #0
        ldr r12, [r12, #VICADDRESS] /* get the address of the handler */
        mov lr, pc
        bx r12 /* branch to the interrupt handler */
        ldmfd sp!, {r0-r3, r12, lr} /* restore the previously saved registers */
        msr cpsr_c, #MODE_IRQ | I_BIT /* switch back to irq mode */
        mov lr, #0
        str lr, [lr, #VICADDRESS] /* acknowledge interrupt */
        ldmfd sp!, {lr} /* load the cpsr from the stack */
        msr spsr_cxsf, lr /* restore the spsr */
        ldmfd sp!, {pc}^ /* go to the next instruction after the interrupt */

fiq_vector:
        b fiq_vector

/* swi call #0 to disable IRQ */
fDisableIRQ:
        swi #0
        bx lr

/* swi call #1 to enable IRQ */
fEnableIRQ:
        swi #1
        bx lr

/* swi call #2 to disable FIQ */
fDisableFIQ:
        swi #2
        bx lr

/* swi call #3 to enable FIQ */
fEnableFIQ:
        swi #3
        bx lr

/* generates a software interrupt with a dynamic parameter. */
fTrap:
        stmfd sp!, {r1, lr} /* save r1 and lr */
        ldr r1, =0xe12fff1e /* bx lr */
        stmfd sp!, {r1} /* save instruction on the stack */
        ldr r1, =0xe8bd4000 /* ldmfd sp!, {lr} */
        stmfd sp!, {r1} /* save instruction on the stack */
        mov r1, #0xef000000 /* swi */
        bic r0, r0, #0xff000000 /* make sure parameter is in correct range */
        orr r0, r1, r0 /* or opcode with parameter */
        stmfd sp!, {r0} /* save the complete swi instruction on the stack */
        ldr r1, =0xe92d4000 /* stmfd sp!, {lr} */
        stmfd sp!, {r1} /* save instruction on the stack */
        mov lr, pc /* save the return address */
        mov pc, sp /* branch to the swi instruction */
        add sp, sp, #16 /* restore the old stack pointer */
        ldmfd sp!, {r1, lr} /* restore r1 and the lr */
        bx lr /* and return */

/* host to network byte order and vice versa */
ntohs:
htons:
        stmfd sp!, {r1, r2}
        mov r1, r0
        mov r2, r0, lsr #8
        and r1, r0, #0xff
        mov r0, r1, lsl #8
        orr r0, r0, r2
        ldmfd sp!, {r1, r2}
        bx lr

ntohl:
htonl:
        stmfd sp!, {r1, r2, r3}
        mov r1, r0
        and r1, r1, #0xff
        mov r2, r0, lsr #8
        and r2, r2, #0xff
        mov r3, r0, lsr #16
        and r3, r3, #0xff
        lsr r0, r0, #24
        lsl r3, r3, #8
        lsl r2, r2, #16
        lsl r1, r1, #24
        orr r0, r0, r3
        orr r0, r0, r2
        orr r0, r0, r1
        ldmfd sp!, {r1, r2, r3}
        bx lr

        end

/******************************************************************************
 * PRIVATE FUNCTIONS (STATIC)
 *****************************************************************************/

/******************************************************************************
 * END OF CODE
 *****************************************************************************/
