#ifndef __LWIPOPTS_H__
#define __LWIPOPTS_H__

typedef unsigned char      u_char;
typedef unsigned short int u_short;
typedef unsigned int       u_int;


#define NO_SYS                          1

/* perform MEMP overflow checks (see memp.c) */
#ifndef NDEBUG
#define MEMP_OVERFLOW_CHECK             2
#endif

/* TODO: The finetuning of MEM_SIZE and TCP options was done with
   trial and error to get an acceptable upload rate. A more precise
   (based on facts) tuning has to be done. CAUTION: TCP_SND_BUF
   shall not exceed MEM_SIZE, otherwise the TCPIP stack may hang.
*/

/* MEM_SIZE: the size of the heap memory. If the application will send
a lot of data that needs to be copied, this should be set high. */
#define MEM_SIZE                        12288

/* ---------- RAW options ---------- */
#define LWIP_RAW                        0
#define RAW_TTL                         255

/* ---------- TCP options ---------- */
#define TCP_MSS                         1500
#define TCP_SND_BUF                     9000
#define MEMP_NUM_PBUF                   64

/* ---------- Pbuf options ---------- */
#define PBUF_LINK_HLEN                  16

/* --------- SNMP options ---------- */
#define LWIP_SNMP                       0

/* ---------- UDP options ---------- */
#define LWIP_UDP                        1
#define UDP_TTL                         255

/* 1 if you want to do an ARP check on the offered address
   (recommended). */
#define DHCP_DOES_ARP_CHECK             0

/* ---------- Statistics options ---------- */
#ifndef LWIP_STATS
#define LWIP_STATS                      0
#endif

#if LWIP_STATS
#define LWIP_STATS_DISPLAY              0
#define LINK_STATS                      1
#define IP_STATS                        1
#define IPFRAG_STATS                    1
#define ICMP_STATS                      1
#define UDP_STATS                       1
#define TCP_STATS                       1
#define MEM_STATS                       1
#define MEMP_STATS                      1
#define PBUF_STATS                      1
#define SYS_STATS                       1
#define RAW_STATS                       0
#endif

/* ---------- PPP options ---------- */

#define PPP_SUPPORT                     0

#if PPP_SUPPORT > 0
#define PPP_THREAD_PRIO                 12
#define PAP_SUPPORT                     1      /* Set for PAP. */
#define UPAP_DEFTIMEOUT                 3
#define CHAP_SUPPORT                    0      /* Set for CHAP. */
#define MD5_SUPPORT                     0      /* Set for MD5 (see also CHAP) */
#define LWIP_ARP                        1      /* disable ARP functionality */
#define ARP_QUEUEING                    0
//TODO: 23.10.07 dont allow vj cause of bug in vj compression
//#define VJ_SUPPORT                      1      /* Set for VJ header compression. */
#define VJ_SUPPORT                      0
#else
#define VJ_SUPPORT                      0      /* Set for VJ header compression. */
#define MD5_SUPPORT                     0      /* Set for MD5 (see also CHAP) */
#define CHAP_SUPPORT                    0      /* Set for CHAP. */
#define PAP_SUPPORT                     0      /* Set for PAP. */
#endif

/* ---------- SLIP options --------- */
#define SLIPIF_THREAD_PRIO              12


/* --------- Debug options --------- */

//hw@truninger.ag: if you intend to use lwip specific assert, you have to use
//the slave control bus on COM3
#define LWIP_NOASSERT                   1

//#define LWIP_DEBUG						LWIP_DBG_OFF
#define DBG_TYPES_ON                    LWIP_DBG_OFF

#define ETHARP_DEBUG                    LWIP_DBG_OFF
#define NETIF_DEBUG                     LWIP_DBG_OFF
#define PBUF_DEBUG                      LWIP_DBG_OFF
#define API_LIB_DEBUG                   LWIP_DBG_OFF
#define API_MSG_DEBUG                   LWIP_DBG_OFF
#define SOCKETS_DEBUG                   LWIP_DBG_OFF
#define ICMP_DEBUG                      LWIP_DBG_OFF
#define INET_DEBUG                      LWIP_DBG_OFF
#define IP_DEBUG                        LWIP_DBG_OFF
#define IP_REASS_DEBUG                  LWIP_DBG_OFF
#define RAW_DEBUG                       LWIP_DBG_OFF
#define MEM_DEBUG                       LWIP_DBG_OFF
#define MEMP_DEBUG                      LWIP_DBG_OFF
#define SYS_DEBUG                       LWIP_DBG_OFF
#define TCP_DEBUG                       LWIP_DBG_OFF
#define TCP_INPUT_DEBUG                 LWIP_DBG_OFF
#define TCP_FR_DEBUG                    LWIP_DBG_OFF
#define TCP_RTO_DEBUG                   LWIP_DBG_OFF
#define TCP_CWND_DEBUG                  LWIP_DBG_OFF
#define TCP_WND_DEBUG                   LWIP_DBG_OFF
#define TCP_OUTPUT_DEBUG                LWIP_DBG_OFF
#define TCP_RST_DEBUG                   LWIP_DBG_OFF
#define TCP_QLEN_DEBUG                  LWIP_DBG_OFF
#define UDP_DEBUG                       LWIP_DBG_OFF
#define TCPIP_DEBUG                     LWIP_DBG_OFF
#define PPP_DEBUG                       LWIP_DBG_OFF
#define SLIP_DEBUG                      LWIP_DBG_OFF
#define DHCP_DEBUG                      LWIP_DBG_OFF
#define TCP_MSG_DEBUG                   LWIP_DBG_OFF
#define DEBUG_SERIO                     LWIP_DBG_OFF

#define DBG_MIN_LEVEL                   DBG_LEVEL_OFF


/* ------- support options --------- */

/* 
Disables netconn by default. Otherwise Visual Studio compiler will produce
preprocessor #error if NO_SYS = 1
*/
#define LWIP_NETCONN 0

/* 
Disables socket by default. Otherwise Visual Studio compiler will produce
preprocessor #error if NO_SYS = 1
*/
#define LWIP_SOCKET 0

/*
Enable DNS resolve functionality. DNS needs as well UDP
*/
#define LWIP_DNS                    1

#define LWIP_DHCP                   1
#define LWIP_IGMP                   1
#define LWIP_HAVE_LOOPIF            1
#define MEMP_NUM_SYS_TIMEOUT        7

#define LWIP_RAND                   randfunc
#endif /* __LWIPOPTS_H__ */
