#include "base_def.h"
#include "lwip/init.h"
#include "lwip/netif.h"
#include "netif/etharp.h"
#include "lwip/stats.h"
#include "lwip/dhcp.h"
#include "lwip/dns.h"
#include "lwip/nbns.h"
#include "netif/ethernetif.h"
#include "lwip/timers.h"
#include "lwip/sntp.h"


static struct netif  eth0_data;
struct netif* eth0 = &eth0_data;
static struct ip_addr my_ipaddr_data;
static struct ip_addr my_netmask_data;
static struct ip_addr my_gw_data;

u32_t n;

void timer(void)
{
  n+=250;
  T0IR = 0xFF;
}

void proc(void *arg, struct udp_pcb *pcb, struct pbuf *p, struct ip_addr *addr, u16_t port)
{
  pbuf_free(p);
}

static struct udp_pcb *conn;

static void sendUDP(void)
{
	static int counter;
	struct pbuf *buf;

	buf = pbuf_alloc(PBUF_TRANSPORT, sizeof(counter), PBUF_RAM);
	memcpy(buf->payload, (void*) &counter, sizeof(counter));
	counter++;
	
	udp_send(conn, buf);
	pbuf_free(buf);
}

u32_t sys_now(void)
{
  return n;
}

volatile enum netif_status netstat;

int main()
{
  fEnableIRQ();
  netstat = status_disconnected;

  lwip_init();
  netif_add(eth0, &my_ipaddr_data, &my_netmask_data, &my_gw_data, (void*)&netstat, ethernetif_init, ethernet_input);
	netif_set_default(eth0);
  
  PCLKSEL0 |= (1 << 2);
  T0TCR = 0;
  T0PR = 0;
  T0MR0 = 18000000;
  T0MCR = 0x03;
  T0TCR = 1;
  VICVectPriority4 = 1;
  VICVectAddr4 = (tDWord)timer;
  VICIntEnable = 0x10;

  dhcp_start(eth0);
  sntp_init();
  nbns_init(eth0);

  //struct ip_addr rip;
	//IP4_ADDR(&rip, 192,168,1,40);
	conn = udp_new();
	udp_connect(conn, IPADDR_ANY, 333);
	udp_bind(conn, NULL, 9999);
	udp_recv(conn, proc, NULL);

  for(;;)
  {
    switch(netstat)
    {
      case status_disconnected:
      {
        break;
      }
      case status_connecting:
      {
        netstat = status_connected;
        sntp_request(NULL);
        dhcp_network_changed(eth0);
        break;
      }
      case status_connected:
      {
        ethernetif_poll(eth0);
        break;
      }
      case status_disconnecting:
      {
        netstat = status_disconnected;
        break;
      }
    }
    
    sys_check_timeouts();
  }
  return 0;
}
